@@ -1,12 +1,12 @@
 //! CRAM reader and record iterator.
 
-pub(crate) mod block;
-pub(crate) mod compression_header;
 mod container;
-mod encoding;
+pub(crate) mod data_container;
+pub(crate) mod num;
 pub(crate) mod record;
 mod records;
-pub(crate) mod slice;
+
+use crate::data_container::DataContainer;
 
 pub use self::records::Records;
 
@@ -17,6 +17,7 @@ use std::{
 
 use byteorder::{LittleEndian, ReadBytesExt};
 
+use self::container::read_container;
 use super::{container::Block, file_definition::Version, Container, FileDefinition, MAGIC_NUMBER};
 
 /// A CRAM reader.
@@ -111,7 +112,7 @@ where
     /// # Ok::<(), io::Error>(())
     /// ```
     pub fn read_file_header(&mut self) -> io::Result<String> {
-        let container = self.read_container()?;
+        let container = read_container(&mut self.inner)?;
 
         if let Some(block) = container.blocks().first() {
             read_file_header_block(block)
@@ -124,17 +125,13 @@ where
     }
 
     pub(crate) fn read_container(&mut self) -> io::Result<Container> {
-        let header = container::read_header(&mut self.inner)?;
+        read_container(&mut self.inner)
+    }
 
-        let blocks_len = header.block_count() as usize;
-        let mut blocks = Vec::with_capacity(blocks_len);
+    pub(crate) fn read_data_container(&mut self) -> io::Result<Option<DataContainer>> {
+        use self::data_container::read_data_container;
 
-        for _ in 0..blocks_len {
-            let block = block::read_block(&mut self.inner)?;
-            blocks.push(block);
-        }
-
-        Ok(Container::new(header, blocks))
+        read_data_container(&mut self.inner)
     }
 
     /// Returns a iterator over records starting from the current stream position.
@@ -238,7 +235,7 @@ where
     Ok(buf)
 }
 
-fn read_file_header_block(block: &Block) -> io::Result<String> {
+pub(crate) fn read_file_header_block(block: &Block) -> io::Result<String> {
     use crate::container::block::ContentType;
 
     if block.content_type() != ContentType::FileHeader {
@@ -266,7 +263,7 @@ fn read_file_header_block(block: &Block) -> io::Result<String> {
 mod tests {
     use std::convert::TryFrom;
 
-    use crate::{container::block::ContentType, num::Itf8};
+    use crate::container::block::ContentType;
 
     use super::*;
 
@@ -302,7 +299,7 @@ mod tests {
 
         let block = Block::builder()
             .set_content_type(ContentType::FileHeader)
-            .set_uncompressed_len(data.len() as Itf8)
+            .set_uncompressed_len(data.len())
             .set_data(data.to_vec())
             .build();
 
