@@ -9,13 +9,15 @@ use byteorder::ReadBytesExt;
 use noodles_bam as bam;
 use noodles_sam as sam;
 
+use super::num::read_itf8;
 use crate::{
-    container::{
+    container::ReferenceSequenceId,
+    data_container::{
         compression_header::{data_series_encoding_map::DataSeries, encoding::Encoding},
-        CompressionHeader, ReferenceSequenceId,
+        CompressionHeader,
     },
     huffman::CanonicalHuffmanDecoder,
-    num::{read_itf8, Itf8},
+    num::Itf8,
     record::{self, feature, tag, Feature, ReadGroupId, Tag},
     BitReader, Record,
 };
@@ -157,7 +159,9 @@ where
                     .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))?
             };
 
-        record.read_length = self.read_read_length()?;
+        record.read_length = self.read_read_length().and_then(|n| {
+            usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
+        })?;
 
         let ap_data_series_delta = self
             .compression_header
@@ -469,10 +473,10 @@ where
         let flags = record.flags();
 
         if flags.are_quality_scores_stored_as_array() {
-            let read_len = record.read_length();
-            let mut scores = Vec::with_capacity(read_len as usize);
+            let read_length = record.read_length();
+            let mut scores = Vec::with_capacity(read_length);
 
-            for _ in 0..read_len {
+            for _ in 0..read_length {
                 let score = self.read_quality_score()?;
                 scores.push(score);
             }
@@ -841,7 +845,9 @@ where
     fn read_unmapped_read(&mut self, record: &mut Record) -> io::Result<()> {
         record.bases.clear();
 
-        for _ in 0..record.read_length() {
+        let read_length = record.read_length();
+
+        for _ in 0..read_length {
             let base = self.read_base()?;
             record.bases.push(base);
         }
@@ -849,10 +855,9 @@ where
         let flags = record.flags();
 
         if flags.are_quality_scores_stored_as_array() {
-            let read_len = record.read_length();
-            let mut scores = Vec::with_capacity(read_len as usize);
+            let mut scores = Vec::with_capacity(read_length);
 
-            for _ in 0..read_len {
+            for _ in 0..read_length {
                 let score = self.read_quality_score()?;
                 scores.push(score);
             }
