@@ -2,6 +2,7 @@ mod tag;
 
 use std::{
     collections::HashMap,
+    convert::TryFrom,
     error, fmt,
     io::{self, Write},
 };
@@ -11,12 +12,14 @@ use byteorder::WriteBytesExt;
 use noodles_bam as bam;
 use noodles_sam as sam;
 
+use super::num::write_itf8;
 use crate::{
-    container::{
+    container::ReferenceSequenceId,
+    data_container::{
         compression_header::{data_series_encoding_map::DataSeries, Encoding},
-        CompressionHeader, ReferenceSequenceId,
+        CompressionHeader,
     },
-    num::{write_itf8, Itf8},
+    num::Itf8,
     record::{self, feature, Feature, Flags, NextMateFlags},
     BitWriter, Record,
 };
@@ -182,17 +185,20 @@ where
             })
     }
 
-    fn write_read_length(&mut self, read_length: Itf8) -> io::Result<()> {
+    fn write_read_length(&mut self, read_length: usize) -> io::Result<()> {
         let encoding = self
             .compression_header
             .data_series_encoding_map()
             .read_lengths_encoding();
 
+        let len = Itf8::try_from(read_length)
+            .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))?;
+
         encode_itf8(
             encoding,
             &mut self.core_data_writer,
             &mut self.external_data_writers,
-            read_length,
+            len,
         )
     }
 
